Package: normr
Type: Package
Title: Normalization and difference calling in ChIP-seq data
Version: 1.39.2
Date: 2026-05-21
Authors@R: c(person(given="Johannes",family="Helmuth",
        email="johannes.helmuth@laborberlin.com", role=c("aut", "cre")),
        person(given="Ho-Ryun",family="Chung",
        email="chung@molgen.mpg.de", role=c("aut")))
Author: Johannes Helmuth [aut, cre], Ho-Ryun Chung [aut]
Maintainer: Johannes Helmuth <johannes.helmuth@laborberlin.com>
Description: Robust normalization and difference calling procedures for
    ChIP-seq and alike data. Read counts are modeled jointly as a binomial
    mixture model with a user-specified number of components. A fitted
    background estimate accounts for the effect of enrichment in certain
    regions and, therefore, represents an appropriate null hypothesis. This
    robust background is used to identify significantly enriched or depleted
    regions.
License: GPL-2
Depends: R (>= 3.3.0)
LinkingTo: Rcpp
SystemRequirements: C++11
Imports: methods, stats, utils, grDevices, parallel, GenomeInfoDb,
        GenomicRanges, IRanges, Rcpp (>= 0.11), qvalue (>= 2.2),
        bamsignals (>= 1.4), rtracklayer (>= 1.32)
Suggests: BiocStyle, testthat (>= 1.0), knitr, rmarkdown
Enhances: BiocParallel
biocViews: Bayesian, DifferentialPeakCalling, Classification,
        DataImport, ChIPSeq, RIPSeq, FunctionalGenomics, Genetics,
        MultipleComparison, Normalization, PeakDetection,
        Preprocessing, Alignment
VignetteBuilder: knitr
NeedsCompilation: yes
Collate: 'NormRFit.R' 'methods.R' 'NormRCountConfig.R' 'RcppExports.R'
RoxygenNote: 6.0.1
URL: https://github.com/your-highness/normR
BugReports: https://github.com/your-highness/normR/issues
git_url: https://git.bioconductor.org/packages/normr
git_branch: devel
git_last_commit: f2ac4d8
git_last_commit_date: 2026-05-21
Repository: Bioconductor 3.24
Date/Publication: 2026-06-01
Packaged: 2026-06-02 00:12:55 UTC; biocbuild
