# Generated by roxygen2: do not edit by hand

export(diffR)
export(enrichR)
export(exportR)
export(getClasses)
export(getCounts)
export(getEnrichment)
export(getPosteriors)
export(getPvalues)
export(getQvalues)
export(getRanges)
export(regimeR)
exportClasses(NormRCountConfig)
exportClasses(NormRFit)
exportMethods(countConfigPairedEnd)
exportMethods(countConfigSingleEnd)
exportMethods(diffR)
exportMethods(enrichR)
exportMethods(exportR)
exportMethods(getClasses)
exportMethods(getCounts)
exportMethods(getEnrichment)
exportMethods(getFilter)
exportMethods(getPosteriors)
exportMethods(getPvalues)
exportMethods(getQvalues)
exportMethods(getRanges)
exportMethods(length)
exportMethods(plot)
exportMethods(print)
exportMethods(regimeR)
exportMethods(show)
exportMethods(summary)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(bamsignals)
import(grDevices)
import(methods)
import(parallel)
importFrom(qvalue,qvalue)
importFrom(rtracklayer,export)
importFrom(stats,na.omit)
importFrom(utils,capture.output)
importFrom(utils,write.table)
useDynLib(normr, .registration=TRUE)
